from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="ngenix",
    asn_patterns=["ngenix"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".cdn.ngenix.net",
            pattern=r"s${random-number}.cdn.ngenix.net",
            source="https://docs.ngenix.net/integraciya-s-platformoi/podklyuchenie-veb-resursa/hto-wsa-cname",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["ngenix"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="netlify",
    asn_patterns=["netlify"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".netlify.app",
            source="https://www.netlify.com/blog/2020/03/26/how-to-set-up-netlify-dns-custom-domains-cname-and-a-records/",
            is_root=True,
            is_leaf=True,
        ),
        CNAMEPattern(
            suffix=".netlify.com",
            source="https://docs.netlify.com/domains-https/custom-domains/configure-external-dns/",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["netlify"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdnsun",
    asn_patterns=["cdnsun"],
    cname_suffixes=[
        CNAMEPattern(suffix=".r.cdn.net"),
        CNAMEPattern(suffix=".cdnsun.net", is_root=True, is_leaf=True),
    ],
    cidr=BGPViewCIDR(["cdnsun"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="adobe",
    asn_patterns=["adobe"],
    cname_suffixes=[
        CNAMEPattern(
            suffix="cdn.adobeaemcloud.com",
            source="https://experienceleague.adobe.com/en/docs/experience-manager-cloud-service/content/implementing/using-cloud-manager/custom-domain-names/configure-dns-settings",
            # ;; ANSWER SECTION:
            # www.otempo.com.br.      10      IN      CNAME   cdn.adobeaemcloud.com.
            # cdn.adobeaemcloud.com.  10      IN      CNAME   adobe-aem.map.fastly.net.
            # adobe-aem.map.fastly.net. 10    IN      A       151.101.111.10
            is_root=True,
            is_leaf=False,
        ),
    ],
    cidr=BGPViewCIDR(["adobe"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class GCoreCIDR(CIDR):
    ipv4_url: str = "https://api.gcore.com/cdn/public-ip-list"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://apidocs.gcore.com/cdn#tag/IP-Addresses-List
        [2] https://api.gcorelabs.com/cdn/public-net-list
        """
        import ipaddress

        data = self.http_get(self.ipv4_url).json()

        ipv4_networks = []
        for ipv4_prefix in data["addresses"]:
            ipv4_networks.append(str(ipaddress.IPv4Network(ipv4_prefix, strict=False)))

        ipv6_networks = []
        for ipv6_prefix in data["addresses_v6"]:
            ipv6_networks.append(str(ipaddress.IPv6Network(ipv6_prefix, strict=False)))

        return {
            "ipv4": "\n".join(ipv4_networks),
            "ipv6": "\n".join(ipv6_networks),
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="gcore",
    asn_patterns=["gcore"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".gcdn.co",
            source="https://www.netify.ai/resources/domains/gcdn.co",
            example="cl-3249f890.gcdn.co",
            is_leaf=True,
        ),
    ],
    cidr=GCoreCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="alibaba-dcdn",
    asn_patterns=["alibaba", "taobao", "alicloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdngslb.com", pattern=r"${domain}.cdngslb.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".m.alikunlun.com", pattern=r"${domain}.m.alikunlun.com", is_leaf=True),
        CNAMEPattern(suffix=".m.alikunlun.net", pattern=r"${domain}.m.alikunlun.net", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunaq.com", pattern=r"${domain}.w.kunlunaq.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunar.com", pattern=r"${domain}.w.kunlunar.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunca.com", pattern=r"${domain}.w.kunlunca.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunluncan.com", pattern=r"${domain}.w.kunluncan.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunea.com", pattern=r"${domain}.w.kunlunea.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlungem.com", pattern=r"${domain}.w.kunlungem.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlungr.com", pattern=r"${domain}.w.kunlungr.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunhuf.com", pattern=r"${domain}.w.kunlunhuf.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunle.com", pattern=r"${domain}.w.kunlunle.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunno.com", pattern=r"${domain}.w.kunlunno.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunpi.com", pattern=r"${domain}.w.kunlunpi.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunsl.com", pattern=r"${domain}.w.kunlunsl.com", is_leaf=True),
        CNAMEPattern(suffix=".w.kunlunso.com", pattern=r"${domain}.w.kunlunso.com", is_leaf=True),
    ],
    cidr=BGPViewCIDR(["alibaba", "taobao", "alicloud"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="verification",
            pattern=r"verify_[0-9a-f]{32}",
        ),
        http=HTTPOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            path="/verification.html",
            pattern=r"verify_[0-9a-f]{32}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



# *.netdna-cdn.com
# *.netdna-ssl.com
# *.netdna.com
from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="netdna",
    asn_patterns=["netdna"],
    cname_suffixes=[
        CNAMEPattern(suffix=".netdna-cdn.com", source="https://github.com/EdOverflow/can-i-take-over-xyz/issues/94"),
    ],
    cidr=BGPViewCIDR(["netdna"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="baidu",
    asn_patterns=["baidu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".a.bdydns.com", pattern=r"${domain}.a.bdydns.com", is_root=True),
        CNAMEPattern(suffix=".yjs-cdn.com", pattern=r"${domain}.yjs-cdn.com", is_root=True),
        CNAMEPattern(suffix=".yunjiasu-cdn.net", pattern=r"${domain}.yunjiasu-cdn.net", is_root=True),
        CNAMEPattern(suffix="opencdnka.jomodns.com", pattern=r"opencdnka.jomodns.com", is_leaf=True),
        CNAMEPattern(suffix="opencdnkav6.jomodns.com", pattern=r"opencdnkav6.jomodns.com", is_leaf=True),
    ],
    cidr=BGPViewCIDR(["baidu"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="bdy-verify",
            pattern=r"[0-9a-f]{8}-[0-9a-f]{8}",
        ),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="leaseweb",
    asn_patterns=["leaseweb"],
    cname_suffixes=[
        CNAMEPattern(suffix=".leasewebultracdn.com", pattern=r"di-[0-9a-z]{8}.leasewebultracdn.com"),
        CNAMEPattern(suffix=".lswcdn.net", pattern=r"di-[0-9a-z]{8}.pr.lswcdn.net"),
        CNAMEPattern(suffix=".llnwi.net", pattern=r"leaseweb.s.llnwi.net", is_leaf=True),
    ],
    cidr=BGPViewCIDR(query_term_list=["leaseweb"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="reblaze",
    asn_patterns=["reblaze"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".rbzdns.com",
            source="https://gb.docs.reblaze.com/v/v2.14/product-walkthrough/settings/dns",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["reblaze"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="edgio",
    asn_patterns=["edgio"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # www.sigmalive.com.      0       IN      CNAME   47d1362e-ac5a-448d-a673-253e6e52c412.glb.edgio.net.
        # 47d1362e-ac5a-448d-a673-253e6e52c412.glb.edgio.net. 3600 IN CNAME tp01.map.edgio.net.
        # tp01.map.edgio.net.     0       IN      A       152.199.6.208
        CNAMEPattern(suffix=".glb.edgio.net", is_root=True, is_leaf=False),
        CNAMEPattern(suffix=".map.edgio.net", is_root=False, is_leaf=True),
    ],
    cidr=BGPViewCIDR(["edgio"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="ucloud",
    asn_patterns=["ucloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".ucloud.com.cn", pattern=r"${domain}.ucloud.com.cn", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".ucloudnaming.cn", pattern=r"${domain}.ucloudnaming.cn"),
        CNAMEPattern(suffix=".ucloudnaming.info", pattern=r"${domain}.ucloudnaming.info", is_leaf=True),
        CNAMEPattern(suffix=".ugslb.net", pattern=r"${domain}.ugslb.net", is_leaf=True),
    ],
    cidr=BGPViewCIDR(query_term_list=["ucloud"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="beluga",
    asn_patterns=["beluga"],
    cname_suffixes=[
        CNAMEPattern(suffix=".belugacdn.com", is_root=True, is_leaf=True),
    ],
    cidr=BGPViewCIDR(["beluga"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification
from cdnmon.util.cidr import deduplicate_networks


class CloudflareCIDR(CIDR):
    ipv4_url: str = "https://www.cloudflare.com/ips-v4"
    ipv6_url: str = "https://www.cloudflare.com/ips-v6"

    def extract(self):
        return {
            "ipv4": self.http_get(self.ipv4_url).text,
            "ipv6": self.http_get(self.ipv6_url).text,
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="cloudflare",
    asn_patterns=["cloudflare"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdn.cloudflare.net", pattern=r"${domain}.cdn.cloudflare.net", is_leaf=True),
    ],
    cidr=CloudflareCIDR(),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="cloudflare-verify",
            pattern=r"[0-9]{8}-[0-9]{8}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="facebook",
    asn_patterns=["facebook", "meta"],
    cname_suffixes=[
        CNAMEPattern(suffix="fbcdn.net", is_leaf=True),
    ],
    cidr=BGPViewCIDR(query_term_list=["facebook", "meta"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="incapsula",
    asn_patterns=["incapsula"],
    cname_suffixes=[
        CNAMEPattern(suffix=".incapdns.net", is_leaf=True),
    ],
    cidr=BGPViewCIDR(["incapsula"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="ucdn",
    asn_patterns=["ucdn"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".b.cdn12.com",
            pattern=r"${random}.b.cdn12.com",
            example="11011-10.b.cdn12.com",
            source="https://help.ucdn.com/how-can-i-add-a-cname-record-to-my-cdn-zone/",
            is_leaf=True,
            is_root=True,
        ),
        CNAMEPattern(
            suffix=".b.cdn13.com",
            pattern=r"${random}.b.cdn13.com",
            example="11011-10.b.cdn13.com",
            source="https://help.ucdn.com/how-can-i-add-a-cname-record-to-my-cdn-zone/",
            is_leaf=True,
            is_root=True,
        ),
    ],
    cidr=BGPViewCIDR(["ucdn"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="inxy",
    asn_patterns=["inxy"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdn.inxy.com", source="https://dev.inxy.hosting/help-center/help-cdnnow/"),
        CNAMEPattern(
            suffix=".hwcdn.net",
            source="https://dev.inxy.hosting/help-center/help-highwinds-cdn/",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["inxy"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="sitelock",
    asn_patterns=["sitelock"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".sitelockcdn.net",
            source="https://www.sitelock.com/help-center/how-to-configure-your-trueshield-web-application-firewall-waf/",
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["sitelock"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification
from cdnmon.util.cidr import deduplicate_networks


class FastlyCIDR(CIDR):
    ipv4_url: str = "https://api.fastly.com/public-ip-list"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://api.fastly.com/public-ip-list
        """
        import ipaddress

        data = self.http_get(self.ipv4_url).json()

        ipv4_networks = []
        for ipv4_prefix in data["addresses"]:
            ipv4_networks.append(str(ipaddress.IPv4Network(ipv4_prefix, strict=False)))

        ipv6_networks = []
        for ipv6_prefix in data["ipv6_addresses"]:
            ipv6_networks.append(str(ipaddress.IPv6Network(ipv6_prefix, strict=False)))

        return {
            "ipv4": "\n".join(ipv4_networks),
            "ipv6": "\n".join(ipv6_networks),
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="fastly",
    asn_patterns=["fastly"],
    cname_suffixes=[
        CNAMEPattern(suffix=".global.ssl.fastly.net", pattern=r"${name}.global.ssl.fastly.net", is_leaf=True),
        CNAMEPattern(suffix=".freetls.fastly.net", pattern=r"${name}.freetls.fastly.net", is_leaf=True),
        CNAMEPattern(suffix=".global.prod.fastly.net", pattern=r"${domain}.global.prod.fastly.net", is_leaf=True),
        CNAMEPattern(suffix=".nonssl.global.fastly.net", pattern=r"nonssl.global.fastly.net", is_leaf=True),
        CNAMEPattern(suffix=".nonssl.us-eu.fastly.net", pattern=r"nonssl.us-eu.fastly.net", is_leaf=True),
        CNAMEPattern(suffix=".global.fastly.net", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".fastly.net"),
    ],
    cidr=FastlyCIDR(),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



import glob
import importlib
import os

from cdnmon.model.cdn import CommonCDN


def CDN(name: str) -> CommonCDN:
    module_name = f"cdnmon.model.cdn.{name.replace('-', '_')}"
    module = importlib.import_module(module_name)
    return module.CDN


cdns = []
glob_pattern = os.path.join(os.path.dirname(__file__), "model", "cdn", "*.py")
for path in glob.glob(glob_pattern):
    cdn_name, _ = os.path.splitext(os.path.basename(path))
    black_list = ["__init__"]
    if cdn_name not in black_list:
        cdns.append(CDN(cdn_name))
cdns = sorted(cdns, key=lambda cdn: cdn.name)

__all__ = ["CDN", "cdns"]



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="jingdong",
    asn_patterns=["jingdong"],
    cname_suffixes=[
        CNAMEPattern(suffix=".360buy.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdinghuo.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdinghuo.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdn.jcloudcache.com", pattern=r"${domain}.lk-[0-9a-f]{6}.cdn.jcloudcache.com"),
        CNAMEPattern(suffix=".cdn.jcloudcdn.com", pattern=r"${domain}.cdn.jcloudcdn.com", is_leaf=True),
        CNAMEPattern(suffix=".cloud-scdn-ns.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-scdn-ns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-scdn-ns.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-scdn-ns.tech", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-scdn.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cloud-starshield.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-starshield.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud-starshield.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".dy.galileo.jcloud-cdn.com", pattern=r"${domain}.dy.galileo.jcloud-cdn.com"),
        CNAMEPattern(suffix=".gslb.qianxun.com", pattern=r"${domain}.gslb.qianxun.com", is_leaf=True),
        CNAMEPattern(suffix=".haiketong.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jcloud-cdn.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jcloudcache.com", pattern=r"${domain}.lk-[0-9a-f]{6}.jcloudcache.com"),
        CNAMEPattern(suffix=".jcloudimg.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".jcloudimg.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jcloudlb.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".jd-eit.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jd.shop", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdn.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdn.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdn.tech", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdndns.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdndns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdndns.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdndns.tech", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdngslb.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdngslb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdngslb.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdnops.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud-scdnops.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloud.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".jdcloudlive.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdcloudstatus.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".jdtxihe.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qianxun.com", source="https://beian.miit.gov.cn/", is_leaf=True),
    ],
    cidr=BGPViewCIDR(["jingdong"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="_cdnautover",
            pattern=r"amNsb3Vk[0-9a-zA-Z]+[=]{0,2}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="byteplus",
    asn_patterns=["byteplus", "bytedance"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # m.filesun.com.          300     IN      CNAME   m.filesun.com.bplslb.com.
        # m.filesun.com.bplslb.com. 120   IN      A       169.150.230.131
        # m.filesun.com.bplslb.com. 120   IN      A       169.150.230.129
        # m.filesun.com.bplslb.com. 120   IN      A       169.150.230.130
        CNAMEPattern(
            suffix=".bplslb.com",
            pattern=r"${domain}.bplslb.com",
            source="https://docs.byteplus.com/zh-CN/docs/byteplus-cdn/docs-set-up-cname",
            is_root=True,
            is_leaf=True,
        ),
        CNAMEPattern(
            suffix="ovx-common.bpltm.com",
            pattern=r"ovx-common.bpltm.com",
            is_root=False,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["byteplus", "bytedance"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="salesforce",
    asn_patterns=["salesforce"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".live.siteforce.com",
            source="https://help.salesforce.com/s/articleView?id=sf.siteforce_domains.htm&type=5",
            is_root=True,
            is_leaf=False,
        ),
        # ;; ANSWER SECTION:
        # support.yahoo-net.jp.   0       IN      CNAME   support.yahoo-net.jp.00d100000002huaeaa.live.siteforce.com.
        # support.yahoo-net.jp.00d100000002huaeaa.live.siteforce.com. 0 IN CNAME 1p.edge2.salesforce.com.
        # 1p.edge2.salesforce.com. 0      IN      CNAME   phoenix-1p.edge2.salesforce.com.
        # phoenix-1p.edge2.salesforce.com. 0 IN   CNAME   ph2.edge2.salesforce.com.
        # ph2.edge2.salesforce.com. 0     IN      A       13.110.52.10
        # ph2.edge2.salesforce.com. 0     IN      A       13.110.52.8
        # ph2.edge2.salesforce.com. 0     IN      A       13.110.52.11
        CNAMEPattern(
            suffix=".edge2.salesforce.com",
            source="https://help.salesforce.com/s/articleView?id=sf.siteforce_domains.htm&type=5",
            is_root=False,
            is_leaf=True,
        ),
        CNAMEPattern(
            suffix=".r.salesforceliveagent.com",
            source="https://help.salesforce.com/s/articleView?id=sf.siteforce_domains.htm&type=5",
            is_root=False,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(query_term_list=["salesforce"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="chinacache",
    asn_patterns=["chinacache"],
    cname_suffixes=[
        # 北京蓝汛通信技术有限责任公司
        CNAMEPattern(suffix=".ccgslb.com"),
        CNAMEPattern(suffix=".lxcvc.com"),
        # ;; ANSWER SECTION:
        # ts1.cn.mm.bing.net.     0       IN      CNAME   ts1.cn.mm.bing.chinacache.net.
        # ts1.cn.mm.bing.chinacache.net. 0 IN     CNAME   ts1.cn.mm.bing.net.bscedge.com.
        # ts1.cn.mm.bing.net.bscedge.com. 0 IN    CNAME   zmxmsn.v.bscedge.com.
        # zmxmsn.v.bscedge.com.   0       IN      CNAME   msnzlxxglobal.v.bscedge.com.
        # msnzlxxglobal.v.bscedge.com. 0  IN      A       4.79.219.67
        # msnzlxxglobal.v.bscedge.com. 0  IN      A       4.79.219.74
        # msnzlxxglobal.v.bscedge.com. 0  IN      A       4.79.219.72
        CNAMEPattern(suffix=".chinacache.net"),
    ],
    cidr=BGPViewCIDR(["chinacache"]),
    homepage="http://chinacache.com/",
    companies=["北京蓝汛通信技术有限责任公司"],
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class CacheFlyCIDR(CIDR):
    ipv4_urls: str = [
        "https://cachefly.execve.workers.dev/ips/cdn.txt",
    ]

    def extract(self):
        """
        [1] https://help.cachefly.com/hc/en-us/articles/215068666-Reverse-Proxy-Source-IPs
        [2] https://cachefly.cachefly.net/ips/rproxy.txt
        [3] https://cachefly.cachefly.net/ips/cdn.txt
        [4] https://cachefly.execve.workers.dev/ips/rproxy.txt
        [5] https://cachefly.execve.workers.dev/ips/cdn.txt
        """
        import ipaddress

        ipv4_networks = []
        for url in self.ipv4_urls:
            data = self.http_get(url).text.strip()
            for line in data.split("\n"):
                ipv4_networks.append(str(ipaddress.IPv4Network(line, strict=False)))

        return {
            "ipv4": "\n".join(ipv4_networks),
            "ipv6": "",
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="cachefly",
    asn_patterns=["cachefly"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".cachefly.net",
            source="https://help.cachefly.com/hc/en-us/articles/215068846-DNS-Configuration-for-CNAMEs-Hostname-Aliases",
            is_leaf=True,
        ),
    ],
    cidr=CacheFlyCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="ctyun",
    asn_patterns=["ctyun", "chinanet"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # pc6.com.                334     IN      CNAME   pc6.com.rgslb.cn.
        # pc6.com.rgslb.cn.       334     IN      CNAME   pc6.com.ctadns.cn.
        # pc6.com.ctadns.cn.      334     IN      A       115.231.173.56
        # pc6.com.ctadns.cn.      334     IN      A       183.136.140.26
        # pc6.com.ctadns.cn.      334     IN      A       183.136.140.25
        # pc6.com.ctadns.cn.      334     IN      A       115.231.173.58
        # pc6.com.ctadns.cn.      334     IN      A       115.231.173.59
        # pc6.com.ctadns.cn.      334     IN      A       115.231.173.57
        # pc6.com.ctadns.cn.      334     IN      A       183.136.140.24
        # pc6.com.ctadns.cn.      334     IN      A       183.136.140.27
        CNAMEPattern(suffix=".ctadns.cn", pattern=r"${domain}.ctadns.cn", is_leaf=True),
        # ;; ANSWER SECTION:
        # mxrb.cn.                0       IN      CNAME   mxrb.cn.ctacdn.cn.
        # mxrb.cn.ctacdn.cn.      0       IN      A       218.67.91.81
        # mxrb.cn.ctacdn.cn.      0       IN      A       106.126.4.84
        CNAMEPattern(suffix=".ctacdn.cn", pattern=r"${domain}.ctacdn.cn", is_root=True, is_leaf=True),
        # ;; ANSWER SECTION:
        # d90.gdl.netease.com.ctlcdn.cn. 592 IN   A       125.77.181.190
        # d90.gdl.netease.com.ctlcdn.cn. 592 IN   A       125.77.181.191
        # d90.gdl.netease.com.ctlcdn.cn. 592 IN   A       125.77.181.192
        CNAMEPattern(suffix=".ctlcdn.cn", pattern=r"${domain}.ctlcdn.cn", is_root=True, is_leaf=True),
        # ;; ANSWER SECTION:
        # storedl5.heytapdownload.com. 0  IN      CNAME   storedl5.heytapdownload.com.ctzcdn.cn.
        # storedl5.heytapdownload.com.ctzcdn.cn. 0 IN A   119.96.5.59
        # storedl5.heytapdownload.com.ctzcdn.cn. 0 IN A   119.96.5.56
        # storedl5.heytapdownload.com.ctzcdn.cn. 0 IN A   119.96.5.58
        # storedl5.heytapdownload.com.ctzcdn.cn. 0 IN A   119.96.5.57
        CNAMEPattern(suffix=".ctzcdn.cn", pattern=r"${domain}.ctzcdn.cn", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".ctyunwaf2.cn"),
        CNAMEPattern(suffix=".ynjtyiptv.cn"),
        CNAMEPattern(suffix=".ahctyiptv.cn"),
        CNAMEPattern(suffix=".ctgcdn.com"),
        CNAMEPattern(suffix=".bjctyiptv.cn"),
        CNAMEPattern(suffix=".shctyiptv.cn"),
        CNAMEPattern(suffix=".jsctyiptv.cn"),
        CNAMEPattern(suffix=".ctydoh.cn"),
        CNAMEPattern(suffix=".fjctyiptv.cn"),
        CNAMEPattern(suffix=".ctfincloud.cn"),
        CNAMEPattern(suffix=".ctyuninner.com"),
        CNAMEPattern(suffix=".ctecdn.cn"),
        CNAMEPattern(suffix=".ctrender.com"),
        CNAMEPattern(suffix=".ctcdnov.com"),
        CNAMEPattern(suffix=".ctyiptv.cn"),
        CNAMEPattern(suffix=".ctcdnov.net"),
        CNAMEPattern(suffix=".ctyunmds.cn"),
        CNAMEPattern(suffix=".ctcdn.com.cn"),
        CNAMEPattern(suffix=".ctyun.com.cn"),
        CNAMEPattern(suffix=".ctdns.com.cn"),
        CNAMEPattern(suffix=".ctxcdn.net"),
        CNAMEPattern(suffix=".ctdns.cn"),
        CNAMEPattern(suffix=".cthcdn.com"),
        CNAMEPattern(suffix=".ctxcdn.cn"),
        CNAMEPattern(suffix=".ctycdn.com"),
        CNAMEPattern(suffix=".ctyun.store"),
        CNAMEPattern(suffix=".ctcdn.com"),
        CNAMEPattern(suffix=".cthcdn.cn"),
        CNAMEPattern(suffix=".ctyunxs.cn"),
        CNAMEPattern(suffix=".cthcdn.net"),
        CNAMEPattern(suffix=".edgecloudx.cn"),
        CNAMEPattern(suffix=".ctzcdn.com"),
        CNAMEPattern(suffix=".ctdcdn.com"),
        CNAMEPattern(suffix=".ctycdn.net.cn"),
        CNAMEPattern(suffix=".ctlcdn.net"),
        CNAMEPattern(suffix=".ctxcdn.com"),
        CNAMEPattern(suffix=".ctycdn.cn"),
        CNAMEPattern(suffix=".ctdns.net"),
        CNAMEPattern(suffix=".ctcdn.cn"),
        CNAMEPattern(suffix=".ctlcdn.com"),
        CNAMEPattern(suffix=".ctmcdn.cn"),
        CNAMEPattern(suffix=".ctyunapi.cn"),
        CNAMEPattern(suffix=".ctbcdn.com"),
        CNAMEPattern(suffix=".ctycdn.net"),
        CNAMEPattern(suffix=".ctyun.cn"),
        CNAMEPattern(suffix=".ctyuncdn.cn"),
    ],
    cidr=BGPViewCIDR(["ctyun", "chinanet"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="dnsverify",
            pattern=r"[0-9]{14}[0-9a-f]{50}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
    homepage="https://www.ctyun.cn/",
    companies=["天翼云科技有限公司"],
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="upyun",
    asn_patterns=["youpai", "upyun"],
    cname_suffixes=[
        CNAMEPattern(suffix=".b0.aicdn.com", pattern=r"[0-9a-f]{12}.b0.aicdn.com", is_root=True),
        CNAMEPattern(suffix="nm.aicdn.com", pattern="nm.aicdn.com", is_leaf=True),
        CNAMEPattern(suffix="vm.ctn.aicdn.com", pattern="vm.ctn.aicdn.com", is_leaf=True),
        CNAMEPattern(suffix="aicdn1.com", pattern="aicdn1.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="aicdn2.com", pattern="aicdn2.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="aicdn3.com", pattern="aicdn3.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="aicdn4.com", pattern="aicdn4.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="aicdn5.com", pattern="aicdn5.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="cdnv1.net", pattern="cdnv1.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="cdnv2.net", pattern="cdnv2.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="cdnv3.net", pattern="cdnv3.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="cdnv4.net", pattern="cdnv4.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="cdnv5.net", pattern="cdnv5.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="oncdp.com", pattern="oncdp.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="sktcdn.com", pattern="sktcdn.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="upai.com", pattern="upai.com", is_leaf=True, source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix="upcdn.net", pattern="upcdn.net", is_leaf=True, source="https://beian.miit.gov.cn/"),
    ],
    cidr=BGPViewCIDR(query_term_list=["youpai", "upyun"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="upyun-verify",
            pattern=r"[0-9a-f]{32}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="wangsu",
    asn_patterns=["wangsu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".6655.la", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".7678.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".8686c.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".90kuai.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".94kuai.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akadns.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akadns88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akadns99.net", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".akamaiedge.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaiedge88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaihd-staging.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaihd-staging88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaihd.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaihd88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaitech.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaitech88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akamaized88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akasrip.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akasrip88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akasripcn.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".akdns.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".allchdlb.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".anjuke.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".battlenet.com.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".bigops.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".bjhlgw.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdn20.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cdn30.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cdn40.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cdn90.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cdn90.org", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdnetgdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cdngtm.cn", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cdnmg.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".cloud-way.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloud365msp.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloudsway.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloudway365.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloudway724.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cloudwaymsp.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cnccdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cncdomain.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cncgdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cncovs.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".cusdvs.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".d4p88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".dlbwscs.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".dnsgtm.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".dnsguest.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".eastday.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ec988.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ecp-cloud.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgekey-staging.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgekey-staging88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgekey.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgekey88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgestreams88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgesuite-staging.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgesuite-staging88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgesuite.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".edgesuite88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".fmsdlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".fmssrc.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".haplat.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".hdtgtm.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".hntncdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".iask.in", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".idnscloud.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".kandianzixun.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxcdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdas.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdfs.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdms.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdns.cn", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(
            suffix=".lxdns.com", pattern=r"${domain}.lxdns.com", source="https://beian.miit.gov.cn/", is_leaf=True
        ),
        CNAMEPattern(suffix=".lxdus.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdvs.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdvsss.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".lxdws.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".maasdk.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".matocloud.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".mileweb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".msp724.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".mspbigops.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".mspcloudway.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".mwcname.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".ngqtlglb.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ogccdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourcdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourcloudsec.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdlbs.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdvs.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdvs.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdvsss.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdvsssvip.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdvsvip.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourdxz.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourfdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourglb0.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourglb0.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourhlb.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourltc.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".oursec1.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourwebat.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourwebcdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourwebcdnvip.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourwebpic.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ourwscs.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ovdlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ovhlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".picdlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".picp.io", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qidian.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlcdn.cn", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".qtlcdn.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".qtlcdn.net", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".qtlcdn360.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlcdn360.top", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlcdn360.xin", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlcdn360.xyz", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlcdncn.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlglb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlgslb.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlgslbcn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlgslbcn.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".qtlgslbcn.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".quantilcn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".quantilcn.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".quantilng.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".scdnga.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".scdnga.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".scdnga.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".scdngc.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".sgccdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".speedcdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".speedws.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".srip.net.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".srip88.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".tl53.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".tl88.net", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".tl88c.net", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".vicp.cc", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".voddlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wangsu.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wangsucloud.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wangsutong.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".waycdn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wbahw.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wcsapi.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wscdn30.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(
            suffix=".wscdns.com", pattern=r"${domain}.wscdns.com", source="https://beian.miit.gov.cn/", is_leaf=True
        ),
        CNAMEPattern(suffix=".wscdnss.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wscloudsec.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wscloudvpn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wscvdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(
            suffix=".wscvip.cn", pattern=r"${domain}.wscvip.cn", source="https://beian.miit.gov.cn/", is_leaf=True
        ),
        CNAMEPattern(suffix=".wscvip.com", pattern=r"${domain}.wscvip.com", is_leaf=True),
        CNAMEPattern(suffix=".wscvip.top", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsdlb.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsdns.top", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(
            suffix=".wsdvs.com", pattern=r"${domain}.wsdvs.com", source="https://beian.miit.gov.cn/", is_leaf=True
        ),
        CNAMEPattern(suffix=".wsglb0.cn", pattern=r"${domain}.wsglb0.cn", is_leaf=True),
        CNAMEPattern(suffix=".wsglb0.com", pattern=r"${domain}.wsglb0.com", is_leaf=True),
        CNAMEPattern(suffix=".wsgtm.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsgtm1.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsgtm2.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsgtm3.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wshttpdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsipv6.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wskam.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsngclouddns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsngcvdns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsnps.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsoss.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsoversea.com", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".wsoversea.info", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wsoversea.net", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".wsoversea.org", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(
            suffix=".wsssec.com", pattern=r"${domain}.wsssec.com", source="https://beian.miit.gov.cn/", is_leaf=True
        ),
        CNAMEPattern(suffix=".wssvip.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wssvs.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wssyun.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".wswebcdn.cn", source="https://beian.miit.gov.cn/", is_leaf=True),
        CNAMEPattern(suffix=".wswebpic.com", pattern=r"${domain}.wswebpic.com", is_leaf=True),
        CNAMEPattern(suffix=".wswebpic.info", source="https://beian.miit.gov.cn/"),
        # ;; ANSWER SECTION:
        # apps.samsung.com.       0       IN      CNAME   apps.gw.samsungapps.com.
        # apps.gw.samsungapps.com. 0      IN      CNAME   apps.samsung.com.gslb.cdnetworks.net.
        # apps.samsung.com.gslb.cdnetworks.net. 7200 IN CNAME apps.samsung.com.wtxcdn.com.
        # apps.samsung.com.wtxcdn.com. 0  IN      A       138.113.26.159
        CNAMEPattern(suffix=".wtxcdn.com", source="https://beian.miit.gov.cn/", is_root=False, is_leaf=True),
        CNAMEPattern(suffix=".yuewen.com", source="https://beian.miit.gov.cn/"),
    ],
    cidr=BGPViewCIDR(query_term_list=["wangsu"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdnetworks",
    asn_patterns=["cdnetworks"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # apps.samsung.com.       0       IN      CNAME   apps.gw.samsungapps.com.
        # apps.gw.samsungapps.com. 0      IN      CNAME   apps.samsung.com.gslb.cdnetworks.net.
        # apps.samsung.com.gslb.cdnetworks.net. 7200 IN CNAME apps.samsung.com.wtxcdn.com.
        # apps.samsung.com.wtxcdn.com. 0  IN      A       138.113.26.159
        CNAMEPattern(suffix=".gslb.cdnetworks.net", is_root=True, is_leaf=False),
    ],
    cidr=BGPViewCIDR(["cdnetworks"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="yottaa",
    asn_patterns=["yottaa"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".yottaa.net",
            pattern=r"${random}.yottaa.net",
            example="fc15a390abd40137aa767e3461d3e37f.yottaa.net",
            source="https://docs.yottaa.com/Content/CNAME%20Usage.htm",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["yottaa"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="tencent",
    asn_patterns=["tencent"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdn.dnsv1.com.cn", pattern=r"${domain}.cdn.dnsv1.com.cn"),
        CNAMEPattern(suffix=".cdn.dnsv1.com", pattern=r"${domain}.cdn.dnsv1.com"),
        CNAMEPattern(suffix=".cdn.qcloudcdn.cn", pattern=r"${domain}.cdn.qcloudcdn.cn"),
        CNAMEPattern(suffix=".dsa.dnsv1.com.cn", pattern=r"${domain}.dsa.dnsv1.com.cn"),
        CNAMEPattern(suffix=".dsa.dnsv1.com", pattern=r"${domain}.dsa.dnsv1.com"),
        CNAMEPattern(suffix=".eo.dnse0.com", pattern=r"${domain}.eo.dnse0.com", is_leaf=True),
        CNAMEPattern(suffix=".eo.dnse1.com", pattern=r"${domain}.eo.dnse1.com", is_leaf=True),
        CNAMEPattern(suffix=".eo.dnse2.com", pattern=r"${domain}.eo.dnse2.com", is_leaf=True),
        CNAMEPattern(suffix=".eo.dnse3.com", pattern=r"${domain}.eo.dnse3.com", is_leaf=True),
        CNAMEPattern(suffix=".eo.dnse4.com", pattern=r"${domain}.eo.dnse4.com", is_leaf=True),
        CNAMEPattern(suffix=".eo.dnse5.com", pattern=r"${domain}.eo.dnse5.com", is_leaf=True),
        CNAMEPattern(suffix=".ovscdns.com", pattern=r"${domain}.ovscdns.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv1.com", pattern=r"${random}.slt-dk.sched.tdnsv1.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv2.com", pattern=r"${random}.slt-dk.sched.tdnsv2.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv3.com", pattern=r"${random}.slt-dk.sched.tdnsv3.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv4.com", pattern=r"${random}.slt-dk.sched.tdnsv4.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv5.com", pattern=r"${random}.slt-dk.sched.tdnsv5.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv6.com", pattern=r"${random}.slt-dk.sched.tdnsv6.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv7.com", pattern=r"${random}.slt-dk.sched.tdnsv7.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv8.com", pattern=r"${random}.slt-dk.sched.tdnsv8.com", is_leaf=True),
        CNAMEPattern(suffix=".slt-dk.sched.tdnsv9.com", pattern=r"${random}.slt-dk.sched.tdnsv9.com", is_leaf=True),
        CNAMEPattern(suffix=".txlivecdn.com", pattern=r"${domain}.txlivecdn.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["tencent"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="_cdnauth",
            pattern=r"[0-9]{14}[0-9a-f]{32}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="frontdoor",
    asn_patterns=["frontdoor", "azure", "microsoft"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # www.fmsc.org.           0       IN      CNAME   fmsc-frontdoor-sc10.azurefd.net.
        # fmsc-frontdoor-sc10.azurefd.net. 0 IN   CNAME   azurefd-t-prod.trafficmanager.net.
        # azurefd-t-prod.trafficmanager.net. 0 IN CNAME   shed.dual-low.s-part-0041.t-0009.t-msedge.net.
        # shed.dual-low.s-part-0041.t-0009.t-msedge.net. 0 IN CNAME s-part-0041.t-0009.t-msedge.net.
        # s-part-0041.t-0009.t-msedge.net. 0 IN   A       13.107.246.69
        CNAMEPattern(suffix=".azureedge.net", pattern=r"${name}.azureedge.net", is_leaf=True),
        CNAMEPattern(suffix=".azurefd.net", pattern=r"${name}.azurefd.net", is_root=True, is_leaf=False),
        CNAMEPattern(suffix=".azurewebsites.net", pattern=r"${name}.azurewebsites.net", is_root=False, is_leaf=True),
        # ;; ANSWER SECTION:
        # www.stateofflorida.com. 0       IN      CNAME   black-stone-001cc220f.5.azurestaticapps.net.
        # black-stone-001cc220f.5.azurestaticapps.net. 0 IN CNAME azurestaticapps5.trafficmanager.net.
        # azurestaticapps5.trafficmanager.net. 0 IN CNAME msha-slice-5-wus2-1.msha-slice-5-wus2-1-ase.p.azurewebsites.net.
        # msha-slice-5-wus2-1.msha-slice-5-wus2-1-ase.p.azurewebsites.net. 0 IN CNAME waws-prod-mwh-5a9d2a3f.sip.p.azurewebsites.windows.net.
        # waws-prod-mwh-5a9d2a3f.sip.p.azurewebsites.windows.net. 0 IN A 20.69.151.16
        CNAMEPattern(suffix=".azurestaticapps.net"),
        CNAMEPattern(suffix=".azurewebsites.windows.net", is_root=False, is_leaf=True),
        # ;; ANSWER SECTION:
        # portal.office.com.      0       IN      CNAME   admin-portal.office.com.
        # admin-portal.office.com. 0      IN      CNAME   portal-office365-com.b-0004.b-msedge.net.
        # portal-office365-com.b-0004.b-msedge.net. 0 IN CNAME b-0004.b-msedge.net.
        # b-0004.b-msedge.net.    0       IN      A       13.107.6.156
        CNAMEPattern(suffix=".b-msedge.net", is_root=False, is_leaf=True),
        # ;; ANSWER SECTION:
        # pkgs.dev.azure.com.     0       IN      CNAME   star-dev-azure-com.l-0011.l-msedge.net.
        # star-dev-azure-com.l-0011.l-msedge.net. 0 IN CNAME l-0011.l-msedge.net.
        # l-0011.l-msedge.net.    0       IN      A       13.107.42.20
        CNAMEPattern(suffix=".l-msedge.net", is_root=True, is_leaf=True),
        # ;; ANSWER SECTION:
        # pacehr.techmahindra.com. 0      IN      CNAME   pacehr-techmahindra.msappproxy.net.
        # pacehr-techmahindra.msappproxy.net. 0 IN CNAME  edf442f5-b994-4c86-a131-b42b03a16c95.tenant.runtime.msappproxy.net.
        # edf442f5-b994-4c86-a131-b42b03a16c95.tenant.runtime.msappproxy.net. 0 IN CNAME cwap-ind1-runtime.routing.msappproxy.net.
        # cwap-ind1-runtime.routing.msappproxy.net. 0 IN CNAME ind.proxy-1.appproxy.msidentity.com.
        # ind.proxy-1.appproxy.msidentity.com. 0 IN CNAME www.tm.ind.proxy-1.appproxy.trafficmanager.net.
        # www.tm.ind.proxy-1.appproxy.trafficmanager.net. 0 IN CNAME proxy-appproxy-ins-ma1p-1.msappproxy.net.
        # proxy-appproxy-ins-ma1p-1.msappproxy.net. 0 IN A 20.44.48.98
        CNAMEPattern(suffix=".msappproxy.net", is_root=True, is_leaf=True),
        # ;; ANSWER SECTION:
        # eastus1-mediap.svc.ms.  0       IN      CNAME   dual-spo-0005.spo-msedge.net.
        # dual-spo-0005.spo-msedge.net. 0 IN      A       13.107.136.10
        # dual-spo-0005.spo-msedge.net. 0 IN      A       13.107.138.10
        CNAMEPattern(suffix=".spo-msedge.net", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".t-msedge.net", is_root=False, is_leaf=True),
        # ;; ANSWER SECTION:
        # cosmic-southafricanorth-ns-eedcdd93b000.trafficmanager.net. 10 IN A 52.123.151.63
        CNAMEPattern(suffix=".trafficmanager.net", is_root=False, is_leaf=True),
    ],
    cidr=BGPViewCIDR(["frontdoor", "azure", "microsoft"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="_dnsauth",
            pattern=r"[0-9a-z]{32}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="iverycloud",
    asn_patterns=["iverycloud"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # pc6.com.                0       IN      CNAME   pc6.com.rgslb.cn.
        # pc6.com.rgslb.cn.       0       IN      CNAME   pc6.com.ctadns.cn.
        # pc6.com.ctadns.cn.      0       IN      A       183.136.140.27
        # pc6.com.ctadns.cn.      0       IN      A       183.136.140.24
        # pc6.com.ctadns.cn.      0       IN      A       115.231.173.57
        # pc6.com.ctadns.cn.      0       IN      A       115.231.173.59
        # pc6.com.ctadns.cn.      0       IN      A       115.231.173.58
        # pc6.com.ctadns.cn.      0       IN      A       183.136.140.25
        # pc6.com.ctadns.cn.      0       IN      A       183.136.140.26
        # pc6.com.ctadns.cn.      0       IN      A       115.231.173.56
        CNAMEPattern(suffix=".rgslb.cn", pattern=r"${domain}.rgslb.cn", is_root=True, is_leaf=False),
    ],
    cidr=BGPViewCIDR(["iverycloud"]),
    homepage="https://www.iverycloud.com/",
    companies=["江苏睿鸿网络技术股份有限公司"],
)



import datetime
import functools
import glob
import ipaddress
import os
import sys
import tempfile
from dataclasses import dataclass
from dataclasses import field
from enum import Enum
from typing import Generator
from typing import List

import humanize
import requests
import yaml
from loguru import logger

from cdnmon.util import bgpview
from cdnmon.util import db
from cdnmon.util.cidr import deduplicate_networks

logger.remove()
logger.add(sys.stderr, level="INFO")


class ETL:
    def extract(self):
        raise NotImplementedError

    def transform(self, data):
        raise NotImplementedError

    def load(self, data):
        raise NotImplementedError


class CIDR(ETL):
    def ipv4_prefixes(self) -> List[str]:
        return self.transform(self.extract())["ipv4_prefixes"]

    def ipv6_prefixes(self) -> List[str]:
        return self.transform(self.extract())["ipv6_prefixes"]

    @functools.lru_cache(maxsize=None)
    def http_get(self, url):
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36 "
        }
        logger.info(f"GET {url}")
        response = requests.get(url, headers=headers)
        logger.info(
            f"{response.status_code} {response.reason} | GET {url}  ({humanize.naturalsize(len(response.content))} Bytes)"
        )
        return response


class BGPViewCIDR(CIDR):
    def __init__(self, query_term_list: List[str] = []):
        self.query_term_list = query_term_list
        self.bgpview_client = bgpview.BGPViewClient()

    def ipv4_prefixes(self) -> List[str]:
        ipv4_networks = []
        for query_term in self.query_term_list:
            data = self.bgpview_client.search(query_term)
            for item in data["data"]["ipv4_prefixes"]:
                ipv4_networks.append(str(ipaddress.IPv4Network(item["prefix"], strict=False)))
        return deduplicate_networks(ipv4_networks, filter_version=4)

    def ipv6_prefixes(self) -> List[str]:
        ipv6_networks = []
        for query_term in self.query_term_list:
            data = self.bgpview_client.search(query_term)
            for item in data["data"]["ipv6_prefixes"]:
                ipv6_networks.append(str(ipaddress.IPv6Network(item["prefix"], strict=False)))
        return deduplicate_networks(ipv6_networks, filter_version=6)


class CNAMEType(Enum):
    """
    e.g. For the following DNS result, the CNAMEType defines as follows

        edgekey.net -> ROOT
        akadns.net  -> INTERMEDIATE
        tl88.net    -> LEAF (which means it directly points to an IP address)

    $ dig www.apple.com

    ; <<>> DiG 9.18.24-0ubuntu5-Ubuntu <<>> www.apple.com
    ;; global options: +cmd
    ;; Got answer:
    ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 24731
    ;; flags: qr rd ra; QUERY: 1, ANSWER: 4, AUTHORITY: 0, ADDITIONAL: 1

    ;; OPT PSEUDOSECTION:
    ; EDNS: version: 0, flags:; udp: 65494
    ;; QUESTION SECTION:
    ;www.apple.com.                 IN      A

    ;; ANSWER SECTION:
    www.apple.com.          0       IN      CNAME   www.apple.com.edgekey.net.
    www.apple.com.edgekey.net. 5050 IN      CNAME   www.apple.com.edgekey.net.globalredir.akadns.net.
    www.apple.com.edgekey.net.globalredir.akadns.net. 0 IN CNAME e6858.e19.s.tl88.net.
    e6858.e19.s.tl88.net.   0       IN      A       106.4.158.58

    ;; Query time: 6 msec
    ;; SERVER: 127.0.0.53#53(127.0.0.53) (UDP)
    ;; WHEN: Mon May 27 15:48:00 CST 2024
    ;; MSG SIZE  rcvd: 187
    """

    UNKNOWN = "unknown"
    ROOT = "root"
    INTERMEDIATE = "intermediate"
    LEAF = "leaf"

    def __str__(self) -> str:
        return self.value


@dataclass
class CNAMEPattern:
    suffix: str = field(default_factory=str)
    pattern: str = field(default_factory=str)
    source: str = field(default_factory=str)
    example: str = field(default_factory=str)
    is_root: bool | None = field(default=None)
    is_leaf: bool | None = field(default=None)
    subscribers: list[str] = field(default_factory=list)

    def __post_init__(self):
        self.suffix = self.suffix if self.suffix.endswith(".") else self.suffix + "."
        if len(self.subscribers) == 0:
            subscribers = [domain for domain in self.yield_subscribers(max_num=5)]
            subscribers.sort()
            self.subscribers = subscribers

    def __str__(self) -> str:
        return self.suffix

    def __repr__(self) -> str:
        return self.suffix

    def marshal(self) -> dict:
        result = {
            "suffix": self.suffix,
            "pattern": self.pattern,
            "source": self.source,
            "subscribers": self.subscribers,
        }
        if self.is_root is not None:
            result["is_root"] = self.is_root
        if self.is_leaf is not None:
            result["is_leaf"] = self.is_leaf
        return result

    def yield_subscribers(self, max_num=8) -> Generator[str, None, None]:
        if not os.getenv("MONGODB_URI"):
            logger.debug("MONGODB_URI is not set")
            return

        suffix = self.suffix if self.suffix.endswith(".") else self.suffix + "."
        pattern = f'^{suffix[::-1].replace(".", chr(92)+".")}'
        filter = {
            "task.dns.response.answers.cname_reverse": {"$regex": pattern},
        }
        collection = db.get_mongo_collection("kepler", "dns")
        cache = set()
        for item in collection.find(filter).batch_size(1):
            if len(cache) >= max_num:
                break
            qname = item["task"]["qname"]
            if qname not in cache:
                logger.warning("{} | {}", qname, self.suffix)
                yield qname
                cache.add(qname)
        return


class DomainOwnershipVerficationStatus(Enum):
    REQUIRED = "required"
    NOT_REQUIRED = "not_required"
    UNKNOWN = "unknown"

    def __str__(self) -> str:
        return self.value


@dataclass
class DomainOwnershipVerification:
    status: DomainOwnershipVerficationStatus = field(default=DomainOwnershipVerficationStatus.UNKNOWN)
    prefix: str = field(default_factory=str)
    pattern: str = field(default_factory=str)

    def __dict__(self) -> dict:
        result = {"status": str(self.status)}
        if self.status == DomainOwnershipVerficationStatus.REQUIRED:
            result["prefix"] = self.prefix
            result["pattern"] = self.pattern
        return result


@dataclass
class HTTPOwnershipVerification:
    status: DomainOwnershipVerficationStatus = field(default=DomainOwnershipVerficationStatus.UNKNOWN)
    path: str = field(default_factory=str)
    pattern: str = field(default_factory=str)

    def __dict__(self) -> dict:
        result = {"status": str(self.status)}
        if self.status == DomainOwnershipVerficationStatus.REQUIRED:
            result["path"] = self.path
            result["pattern"] = self.pattern
        return result


@dataclass
class OwnershipVerification:
    txt: DomainOwnershipVerification = field(default_factory=DomainOwnershipVerification)
    http: HTTPOwnershipVerification = field(default_factory=HTTPOwnershipVerification)

    def __dict__(self) -> dict:
        result = {}
        if self.txt.status != DomainOwnershipVerficationStatus.UNKNOWN:
            result["txt"] = self.txt.__dict__()
        if self.http.status != DomainOwnershipVerficationStatus.UNKNOWN:
            result["http"] = self.http.__dict__()
        return result


@dataclass
class CommonCDN:
    name: str = field(default_factory=str)
    asn_patterns: list[str] = field(default_factory=list)
    cname_suffixes: list[CNAMEPattern] = field(default_factory=list)
    cidr: CIDR = field(default_factory=CIDR)
    frontend_ownership_verification: OwnershipVerification = field(default_factory=OwnershipVerification)
    backend_ownership_verification: OwnershipVerification = field(default_factory=OwnershipVerification)
    homepage: str = field(default_factory=str)
    companies: list[str] = field(default_factory=list)

    def ipv4_prefixes(self) -> List[str]:
        return self.cidr.ipv4_prefixes()

    def ipv6_prefixes(self) -> List[str]:
        return self.cidr.ipv6_prefixes()

    def subscribers(self) -> List[str]:
        results = set()
        for cname_suffix in self.cname_suffixes:
            for subscriber in cname_suffix.yield_subscribers():
                results.add(subscriber)
        return list(results)

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return self.name

    def update(self) -> bool:
        url = f"https://cdnmon.vercel.app/{self.name}.yaml"
        logger.info(f"GET {url}")
        response = requests.get(url)
        if response.status_code == 200:
            data = yaml.safe_load(response.text)
            self.asn_patterns = data["asn_patterns"]
            self.cname_suffixes = [CNAMEPattern(**i) for i in data["cname_suffixes"]]
            self.frontend_ownership_verification = OwnershipVerification(**data["frontend_ownership_verification"])
            self.backend_ownership_verification = OwnershipVerification(**data["backend_ownership_verification"])
            return True
        else:
            return False

    def dump(self) -> bool:
        path = os.path.join(
            os.path.dirname(
                os.path.dirname(
                    os.path.dirname(
                        os.path.dirname(__file__),
                    ),
                ),
            ),
            "assets",
            "cdn",
            f"{self.name}.yaml",
        )

        old_content = ""
        if os.path.exists(path):
            with open(path, mode="r", encoding="utf-8") as f:
                old_content = f.read()

        with tempfile.NamedTemporaryFile(mode="w", encoding="utf-8", delete=False) as f:
            temporary_filename = f.name
            yaml.dump(
                {
                    "name": self.name,
                    "asn_patterns": self.asn_patterns,
                    "cname_suffixes": [i.marshal() for i in self.cname_suffixes],
                    "ipv4_prefixes": self.ipv4_prefixes(),
                    "ipv6_prefixes": self.ipv6_prefixes(),
                    "frontend_ownership_verification": self.frontend_ownership_verification.__dict__(),
                    "backend_ownership_verification": self.backend_ownership_verification.__dict__(),
                    "updated_at": datetime.datetime.now().isoformat(),
                },
                f,
            )

        with open(temporary_filename, mode="r", encoding="utf-8") as f:
            new_content = f.read()

        old_lines = [line for line in old_content.splitlines() if not line.startswith("updated_at:")]
        new_lines = [line for line in new_content.splitlines() if not line.startswith("updated_at:")]

        if old_lines != new_lines:
            # move temporary file to the original file
            os.replace(temporary_filename, path)
            return True
        else:
            # remove temporary file
            os.remove(temporary_filename)
            return False


__all__ = []
for path in glob.glob(f"{os.path.dirname(__file__)}/*.py"):
    name, _ = os.path.splitext(os.path.basename(path))
    if name != "__init__":
        __all__.append(name)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="verizon",
    asn_patterns=["verizon"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(query_term_list=["verizon"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="hubspot",
    asn_patterns=["hubspot"],
    cname_suffixes=[
        CNAMEPattern(suffix=".hubspot.net", source="https://www.hubspot.com/products/cms/cdn", is_leaf=False),
        CNAMEPattern(suffix=".sites.hscoscdn00.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdn10.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdn20.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdn30.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdn40.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdnqa00.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdnqa10.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdnqa20.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdnqa30.net", is_leaf=True),
        CNAMEPattern(suffix=".sites.hscoscdnqa40.net", is_leaf=True),
    ],
    cidr=BGPViewCIDR(query_term_list=["hubspot"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="bunny",
    asn_patterns=["bunny"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".b-cdn.net",
            source="https://support.bunny.net/hc/en-us/articles/207790279-How-to-set-up-a-custom-CDN-hostname",
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["bunny"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="dnion",
    asn_patterns=["dnion"],
    cname_suffixes=[
        CNAMEPattern(suffix=".ayhtbj.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ayhtbj.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".dnion.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".dnion.net", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".globalcdn.com.cn", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".goidns.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".grdunion.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".mcadn.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".prdxg.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".ttxshy.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".tvayht.com", source="https://beian.miit.gov.cn/"),
        CNAMEPattern(suffix=".tvayht.net", source="https://beian.miit.gov.cn/"),
    ],
    cidr=BGPViewCIDR(["dnion"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdn77",
    asn_patterns=["cdn77"],
    cname_suffixes=[
        # web.webpushs.com.       0       IN      CNAME   1317109900.rsc.cdn77.org.
        # 1317109900.rsc.cdn77.org. 0     IN      A       89.187.187.15
        # 1317109900.rsc.cdn77.org. 0     IN      A       143.244.51.207
        # 1317109900.rsc.cdn77.org. 0     IN      A       89.187.187.20
        # 1317109900.rsc.cdn77.org. 0     IN      A       143.244.51.200
        # 1317109900.rsc.cdn77.org. 0     IN      A       89.187.187.11
        # 1317109900.rsc.cdn77.org. 0     IN      A       143.244.51.8
        # 1317109900.rsc.cdn77.org. 0     IN      A       143.244.51.249
        CNAMEPattern(
            suffix=".rsc.cdn77.org",
            source="https://client.cdn77.com/support/knowledgebase/cdn-resource/configuring-cname",
            is_root=True,
            is_leaf=True,
        ),
        CNAMEPattern(suffix=".cdn77.net"),
    ],
    cidr=BGPViewCIDR(["cdn77"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="xiaomi",
    asn_patterns=["xiaomi"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # appstore.cdn.pandora.xiaomi.com. 0 IN   CNAME   appstore.cdn.pandora.c.mgslb.com.
        # appstore.cdn.pandora.c.mgslb.com. 0 IN  CNAME   appstore.cdn.pandora.xiaomi.com.w.alikunlun.com.
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.234
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.231
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.229
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.227
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.232
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.228
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.233
        # appstore.cdn.pandora.xiaomi.com.w.alikunlun.com. 0 IN A 121.194.7.230
        # ;; ANSWER SECTION:
        # gimg.cdn.pandora.xiaomi.com. 0  IN      CNAME   gimg.cdn.pandora.c.mgslb.com.
        # gimg.cdn.pandora.c.mgslb.com. 0 IN      CNAME   gimg.cdn.pandora.xiaomi.com.download.ks-cdn.com.
        # gimg.cdn.pandora.xiaomi.com.download.ks-cdn.com. 0 IN CNAME k1.gslb.ksyuncdn.com.
        # k1.gslb.ksyuncdn.com.   0       IN      A       61.170.102.10
        # k1.gslb.ksyuncdn.com.   0       IN      A       183.61.168.3
        # k1.gslb.ksyuncdn.com.   0       IN      A       183.131.56.3
        # k1.gslb.ksyuncdn.com.   0       IN      A       183.131.40.7
        # k1.gslb.ksyuncdn.com.   0       IN      A       113.16.211.1
        # k1.gslb.ksyuncdn.com.   0       IN      A       61.240.218.6
        CNAMEPattern(suffix=".mgslb.com", pattern=r"${domain}.mgslb.com", is_root=True, is_leaf=False),
    ],
    cidr=BGPViewCIDR(["xiaomi"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="azion",
    asn_patterns=["azion"],
    cname_suffixes=[
        CNAMEPattern(suffix=".map.azionedge.net", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".azioncdn.net", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".azioncdn.com"),
        CNAMEPattern(suffix=".azion.net"),
    ],
    cidr=BGPViewCIDR(["azion"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="netease",
    asn_patterns=["netease"],
    cname_suffixes=[
        CNAMEPattern(suffix=".163jiasu.com", pattern=r"${domain}.163jiasu.com"),
        CNAMEPattern(suffix=".ntes53.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["netease"]),
)



#
from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="blazingcdn",
    asn_patterns=["blazing"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".blazingcdn.net",
            pattern=r"cdn${random-number}.blazingcdn.net",
            example="cdn55449950.blazingcdn.net",
            source="https://knowledgebase.blazingcdn.com/get-started-complete-guide/",
            is_root=True,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["blazingcdn"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="lumen",
    asn_patterns=["lumen"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # www.bellacanvas.com.    0       IN      CNAME   www.bellacanvas.com.c.section.io.
        # www.bellacanvas.com.c.section.io. 0 IN  A       45.154.183.126
        CNAMEPattern(
            suffix=".c.section.io",
            source="https://www.lumen.com/help/en-us/cdn/application-delivery-solutions/set-up-your-domain/set-up-dns.html",
            is_root=True,
            is_leaf=True,
        ),
        CNAMEPattern(
            suffix=".e.ns1.sectionedge.com",
            is_root=False,
            is_leaf=False,
        ),
        # ;; ANSWER SECTION:
        # tsjybcd.com.            0       IN      CNAME   tsjybcd.com.c.section.io.
        # tsjybcd.com.c.section.io. 0     IN      CNAME   dp4dc74tninedmfkv6humrksrmlqidfm.e.ns1.sectionedge.com.
        # dp4dc74tninedmfkv6humrksrmlqidfm.e.ns1.sectionedge.com. 0 IN CNAME lmn-lax-k1.ep.section.io.
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.135
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.45
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.132
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.42
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.37
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.38
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.44
        # lmn-lax-k1.ep.section.io. 0     IN      A       207.120.32.39
        CNAMEPattern(
            suffix=".ep.section.io",
            is_root=False,
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["lumen"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdnvideo",
    asn_patterns=["cdnvideo"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".a.trbcdn.net",
            source="https://doc.cdnvideo.ru/en/Services_setup/http/",
            is_leaf=True,
        ),
    ],
    cidr=BGPViewCIDR(["cdnvideo"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="qiniu",
    asn_patterns=["qiniu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".qiniudns.com", pattern=r"${dot2dash-domain}-id[0-9a-z]{5}.qiniudns.com", is_leaf=True),
        CNAMEPattern(suffix=".hiecheimaetu.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".mikubkt.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".mikuhost.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniumiku.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".mikuio.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".mikudns.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".mikuapi.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qnvic.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qnvic.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuvic.cn", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuvic.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuio.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuqcdn.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qnqcdn.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuqcdn.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniup.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qnydns.com", source="https://beian.miit.gov.cn", is_leaf=True),
        CNAMEPattern(suffix=".niuclass.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniukodo.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".everbox.net", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuapp.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniublob.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniuinc.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniu.us", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qnssl.com", source="https://beian.miit.gov.cn"),
        CNAMEPattern(suffix=".qiniudns2.com", source="https://beian.miit.gov.cn"),
    ],
    cidr=BGPViewCIDR(query_term_list=["qiniu"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="verification",
            pattern=r"verify_[0-9a-f]{33}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="limelight",
    asn_patterns=["limelight"],
    cname_suffixes=[
        # ;; ANSWER SECTION:
        # oqesn.np.dl.playstation.net. 0  IN      CNAME   l02.cdn.update.playstation.net.
        # l02.cdn.update.playstation.net. 0 IN    CNAME   sonycoment.vo.llnwd.net.
        # sonycoment.vo.llnwd.net. 0      IN      A       68.142.107.1
        # sonycoment.vo.llnwd.net. 0      IN      A       68.142.107.129
        CNAMEPattern(suffix=".llnwd.net", is_root=True, is_leaf=True, example="oqesn.np.dl.playstation.net"),
        # ;; ANSWER SECTION:
        # home.bt.com.            0       IN      CNAME   btgrpsec-dd.lldns.net.
        # btgrpsec-dd.lldns.net.  0       IN      CNAME   btgroup-1.s.llnwi.net.
        # btgroup-1.s.llnwi.net.  0       IN      A       68.142.107.88
        CNAMEPattern(suffix=".llnwi.net", is_root=False, is_leaf=True, example="home.bt.com"),
        # ;; ANSWER SECTION:
        # kms.samsfmm.cust.lldns.net. 0   IN      A       69.28.162.131
        CNAMEPattern(suffix=".lldns.net", is_root=True, is_leaf=True, example="home.bt.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["limelight"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="keycdn",
    asn_patterns=["keycdn"],
    cname_suffixes=[
        CNAMEPattern(
            suffix=".kxcdn.com",
            source="https://www.keycdn.com/support/cdn-integration",
            is_leaf=True,
            is_root=True,
        ),
        CNAMEPattern(suffix=".kvcdn.com"),
    ],
    cidr=BGPViewCIDR(["keycdn"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="alibaba-waf",
    asn_patterns=["alibaba", "taobao", "alicloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".yundunwaf1.com", pattern=r"${random}.yundunwaf1.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".yundunwaf2.com", pattern=r"${random}.yundunwaf2.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".yundunwaf3.com", pattern=r"${random}.yundunwaf3.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".yundunwaf4.com", pattern=r"${random}.yundunwaf4.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix=".yundunwaf5.com", pattern=r"${random}.yundunwaf5.com", is_root=True, is_leaf=True),
    ],
    cidr=BGPViewCIDR(["alibaba", "taobao", "alicloud"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="wafdnscheck",
            pattern=r"[0-9]{16}-[0-9a-z]{32}",
        )
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="lumen",
    asn_patterns=["lumen"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(query_term_list=["lumen"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="baishan",
    asn_patterns=["baishan"],
    cname_suffixes=[
        # 北京白山耘科技有限公司
        CNAMEPattern(suffix=".baishan-cloud.com"),
        CNAMEPattern(suffix=".baishancloud.com"),
        CNAMEPattern(suffix=".baishangeek.cn", is_leaf=True),
        CNAMEPattern(suffix=".baishangeek.com"),
        CNAMEPattern(suffix=".bsclink.cn", is_leaf=True),
        CNAMEPattern(suffix=".bsclink.com"),
        CNAMEPattern(suffix=".bscstorage.com"),
        CNAMEPattern(suffix=".bsgslb.cn", pattern=r"${domain}.bsgslb.cn", is_leaf=True),
        CNAMEPattern(suffix=".bsgslb.com", is_leaf=True),
        CNAMEPattern(suffix=".qianxun.com", is_leaf=True),
        CNAMEPattern(suffix=".qingcdn.com", is_leaf=True),
        CNAMEPattern(suffix=".trpcdn.com", is_leaf=True),
        CNAMEPattern(suffix=".trpcdn.net", pattern=r"${domain}.bsgslb.cn"),
        # 贵州白山云科技股份有限公司
        CNAMEPattern(suffix=".5gcdnx.com"),
        CNAMEPattern(suffix=".baicdnx.com"),
        CNAMEPattern(suffix=".bainsx.cn"),
        CNAMEPattern(suffix=".baishan-cloud.net"),
        CNAMEPattern(suffix=".baishan-gateway.com"),
        CNAMEPattern(suffix=".baishan.com"),
        CNAMEPattern(suffix=".baishanyun.com"),
        CNAMEPattern(suffix=".bsccdn.net"),
        CNAMEPattern(suffix=".bscdns.cn"),
        CNAMEPattern(suffix=".bscdns.com"),
        CNAMEPattern(suffix=".v.bscedge.com", is_root=True, is_leaf=True),
        CNAMEPattern(suffix="global.v.bscedge.com", is_root=False, is_leaf=True),
        CNAMEPattern(suffix=".bscgslb.com"),
        CNAMEPattern(suffix=".bsydns.cn"),
        CNAMEPattern(suffix=".bsydns.com"),
        CNAMEPattern(suffix=".bsydns.net"),
        CNAMEPattern(suffix=".ccedge.cn"),
        CNAMEPattern(suffix=".cncdnx.com"),
        CNAMEPattern(suffix=".crazyrtc.com"),
        CNAMEPattern(suffix=".dolfincdnx.net"),
        CNAMEPattern(suffix=".dolfindnsx.top"),
        CNAMEPattern(suffix=".eccdnx.com"),
        CNAMEPattern(suffix=".edgefn.net"),
        CNAMEPattern(suffix=".fnedge.net"),
        CNAMEPattern(suffix=".ldgslb.com"),
        CNAMEPattern(suffix=".scdntop.com"),
        CNAMEPattern(suffix=".whitecdnx.com"),
        CNAMEPattern(suffix=".zzcdnx.com"),
    ],
    cidr=BGPViewCIDR(["baishan"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn import DomainOwnershipVerficationStatus
from cdnmon.model.cdn import DomainOwnershipVerification
from cdnmon.model.cdn import HTTPOwnershipVerification
from cdnmon.model.cdn import OwnershipVerification

CDN = CommonCDN(
    name="kingsoft",
    asn_patterns=["kingsoft", "ksyun"],
    cname_suffixes=[
        CNAMEPattern(suffix=".download.ks-cdn.com", pattern=r"${domain}.download.ks-cdn.com", is_root=True),
        CNAMEPattern(suffix=".ksyuncdn.com", is_leaf=True),
    ],
    cidr=BGPViewCIDR(["kingsoft", "ksyun"]),
    frontend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            prefix="ksy-cdnauth",
            pattern=r"[0-9a-f]{32}",
        ),
        http=HTTPOwnershipVerification(
            status=DomainOwnershipVerficationStatus.REQUIRED,
            path="/ksy-cdnauth.html",
            pattern=r"[0-9a-f]{32}",
        ),
    ),
    backend_ownership_verification=OwnershipVerification(
        txt=DomainOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
        http=HTTPOwnershipVerification(status=DomainOwnershipVerficationStatus.NOT_REQUIRED),
    ),
)
